#
# From work of David Schuetz and Joël Franusic
# https://github.com/dschuetz/1password
# https://github.com/jpf/okta-jwks-to-pem

from sys import exit as sys_exit
from base64 import b64decode as base64_b64decode, urlsafe_b64decode, b64encode, urlsafe_b64decode, urlsafe_b64encode
from binascii import a2b_hex as binascii_a2b_hex
from json import loads as json_loads
from cryptography.hazmat.primitives.ciphers.algorithms import AES
from cryptography.hazmat.primitives.ciphers.modes import GCM
from cryptography.hazmat.primitives.ciphers.base import Cipher
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicNumbers, RSAPrivateNumbers
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.asymmetric import padding
import six
import struct


def aes_decrypt(ct, key, iv, tag):
    cipher = Cipher(AES(key), GCM(iv), default_backend())
    decryptor = cipher.decryptor()
    return decryptor.update(ct) + decryptor.finalize_with_tag(tag)


def intarr2long(arr):
    return int(''.join(["%02x" % byte for byte in arr]), 16)


def base64_to_long(data):
    if isinstance(data, six.text_type):
        data = data.encode("ascii")

    # urlsafe_b64decode will happily convert b64encoded data
    _d = urlsafe_b64decode(bytes(data) + b'==')
    return intarr2long(struct.unpack('%sB' % len(_d), _d))


def rsa_decrypt(key_raw, ct):
    jwk = json_loads(key_raw)
    public_numbers = RSAPublicNumbers(
        base64_to_long(jwk['e']),
        base64_to_long(jwk['n'])
    )
    private_numbers = RSAPrivateNumbers(
        base64_to_long(jwk['p']),
        base64_to_long(jwk['q']),
        base64_to_long(jwk['d']),
        base64_to_long(jwk['dp']),
        base64_to_long(jwk['dq']),
        base64_to_long(jwk['qi']),
        public_numbers
    )
    private_key = private_numbers.private_key(backend=default_backend())

    pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption()
    )
    decryptor = serialization.load_pem_private_key(
        pem,
        password=None,
        backend=default_backend()
    )
    plain = decryptor.decrypt(
        get_binary_from_string(ct),
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA1()),
            algorithm=hashes.SHA1(),
            label=None
        )
    )
    return plain


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Convenience functions for input/output
#
# * opb64d, opb64e - base64 decode with 1Password tricks
#    (URL safe altchars, not always including == padding, etc.)
# 
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


#
# The strings stored by 1Password don't always have padding characters at the
#   end. So we try multiple times until we get a good result.
#
# Also, 1Password uses url-safe encoding with - and _ replacing + and /.
#
def opb64d(b64dat):
    try:
        out = base64_b64decode(b64dat, altchars='-_')
    except:
        try:
            out = base64_b64decode(b64dat + '=', altchars='-_')
        except:
            try:
                out = base64_b64decode(b64dat + '==', altchars='-_')
            except:
                print("Problem b64 decoding string: %s" % (b64dat))
                sys_exit(1)
    return out


def get_binary_from_string(str):
    try:
        bin = binascii_a2b_hex(str)
    except:
        try:
            bin = opb64d(str)
        except:
            try:
                bin = base64_b64decode(str)
            except:
                print("Unable to decode the input. Enter in hex or base64_")
                sys_exit(1)
    return bin


def determine_session_file_path_from_session_key(session_key):
    decoded = urlsafe_b64decode(session_key + '==')
    sha1 = hashes.Hash(hashes.SHA1(), backend=default_backend())
    sha1.update(decoded)
    digest = sha1.finalize()
    return '.' + urlsafe_b64encode(digest).decode('utf-8')[:-1]


from sys import exit
from onepassword_local_search.exceptions.Hook import exception_handler
from onepassword_local_search.exceptions.ManagedException import ManagedException
from onepassword_local_search.OnePassword import OnePassword
from argparse import ArgumentParser, Namespace
import sys
sys.excepthook = exception_handler


class CliSimple:

    onePassword: OnePassword
    args: Namespace
    command: str

    def __init__(self, *args):

        parser = ArgumentParser()
        subparsers = parser.add_subparsers(dest='command')

        action_get = subparsers.add_parser('get')
        action_get.add_argument('uuid', help='uuid to fetch')
        action_get.add_argument('field', help='field to retrieve', nargs='?', default=None)
        action_get.add_argument('--use-custom-uuid', help='use custom UUID mapping', nargs='?', default=False, const=True)
        action_get.add_argument('--use-lastpass-uuid', help='use LastPass UUID mapping', nargs='?', default=False, const=True)

        action_list = subparsers.add_parser('list')
        action_list.add_argument('--format', help='custom format string')
        action_list.add_argument('--filter', help='filter over title entry')

        action_is_authenticated = subparsers.add_parser('is-authenticated')

        action_version = subparsers.add_parser('version')

        action_update_mapping = subparsers.add_parser('mapping')
        action_update_mapping.add_argument('subcommand', help='update or list', choices=['list', 'update'])
        action_update_mapping.add_argument('--use-lastpass-uuid', help='list using lastpass uuid', nargs='?', default=False, const=True)

        self.args = parser.parse_args(args[1:])

    def run(self):
        try:
            if self.args.command == 'version':
                return self.version()
            custom_uuid_mapping = None
            if hasattr(self.args, 'use_custom_uuid') and self.args.use_custom_uuid:
                custom_uuid_mapping = 'UUID'
            elif hasattr(self.args, 'use_lastpass_uuid') and self.args.use_lastpass_uuid:
                custom_uuid_mapping = 'LASTPASS'
            self.onePassword = OnePassword(custom_uuid_mapping=custom_uuid_mapping)
            return getattr(self, self.args.command.replace('-', '_'))()
        except ManagedException as e:
            exit(e.args[0])
        except (BrokenPipeError, IOError):
            sys.stderr.close()
            pass

    def get(self):
        if self.args.use_custom_uuid:
            custom_uuid_mapping = 'UUID'
        elif self.args.use_lastpass_uuid:
            custom_uuid_mapping = 'LASTPASS'
        else:
            custom_uuid_mapping = None
        return self.onePassword.get(self.args.uuid, self.args.field, custom_uuid_mapping)

    def list(self):
        return self.onePassword.list(self.args.format, self.args.filter)

    def mapping(self):
        return self.onePassword.mapping(self.args.subcommand, self.args.use_lastpass_uuid)

    def version(self):
        return OnePassword.version()

    def is_authenticated(self):
        if not self.onePassword.is_authenticated():
            sys.exit(1)




from json import loads as json_loads


class Cipher:

    iv: str
    data: str
    json: dict
    enc: str

    def __init__(self, json_string):
        self.json = json_loads(json_string)
        self.iv = self.json['iv']
        self.data = self.json['data']
        self.enc = self.json['enc']









import pytest
from onepassword_local_search.services.CryptoService import CryptoService
from onepassword_local_search.services.SecondaryCryptoService import SecondaryCryptoService
from onepassword_local_search.services.ConfigFileService import ConfigFileService
from onepassword_local_search.tests.fixtures_common import common_data, op_session, op_personal_session, op_dual_session
from onepassword_local_search.services.StorageService import StorageService
from onepassword_local_search.models.Cipher import Cipher
from onepassword_local_search.models.Item import Item
import os
from json import loads as json_loads


@pytest.mark.usefixtures("op_session")
@pytest.fixture
def crypto_service(mocker):
    return CryptoService(StorageService(), ConfigFileService(), 1)


@pytest.mark.usefixtures("op_personal_session")
@pytest.fixture
def crypto_service_personal(mocker):
    return SecondaryCryptoService(StorageService(), ConfigFileService(), 2)


@pytest.mark.usefixtures("op_dual_session")
def test_personal_session_keys(crypto_service_personal):
    crypto_service = crypto_service_personal

    crypto_service.sessionKey = crypto_service._get_session_key()
    assert crypto_service.sessionKey == common_data('personal_session_key')

    crypto_service.encryptedSessionPrivateKey = crypto_service._get_encrypted_session_key()
    assert type(crypto_service.encryptedSessionPrivateKey).__name__ == 'Cipher'
    assert crypto_service.encryptedSessionPrivateKey.iv == 'Ezo-wY3lGOBPdxBP'
    assert crypto_service.encryptedSessionPrivateKey.data == 'GRT2axfpOiG967Pf7_EMSgPgydko3FuzvTEBry6J_rvW9E1Dp9khrF8wgryBEfei1O8Sl1onyU_u68tB_ws6ux5UWVY1URu95oiMkNV0DzbhoVzITz1mWtr3la73TQjEUjTp8eDHzcjUzvaNcvLw9g-kCD6kDFul5wWzeDlLJm2AgIsaqLKycHgDSk--_c1xQiXxH2_29e0yAibcpXoVp9-Xq3oRranqPH8dEMsybA4ABGLmTKvJJ6Ssv4ALeOEdVFj9B-_V7deMbj1W14uz4f036rGXXwLFVEka6Ca0IPLewbeQFTUlo-B18JKhoSaLVMRrsz9jSokFUuGFaCFQNIs-5w'
    assert crypto_service.encryptedSessionPrivateKey.enc == 'A256GCM'

    crypto_service.sessionPrivateKey = json_loads(crypto_service.decrypt(crypto_service.sessionKey, crypto_service.encryptedSessionPrivateKey))
    assert crypto_service.sessionPrivateKey['encodedMuk'] == 'j4mKCHJyeG-O187LjWvjfYciZnKWrDZ8MjS6jhBNGnc'
    assert crypto_service.sessionPrivateKey['encodedKey'] == '-lPNvxyf2MC6yQcTysBzJzT70UnvbHFrghcYmTPbeEU'

    crypto_service.encryptedSymmetricyKey = Cipher(crypto_service._get_encrypted_symmetric_key())
    assert type(crypto_service.encryptedSymmetricyKey).__name__ == 'Cipher'
    assert crypto_service.encryptedSymmetricyKey.iv == 'tYa8RNJ7UpxA8AXW'
    assert crypto_service.encryptedSymmetricyKey.data == 'HKhbbcnYTeA3Fm1AIdHE5B5N6tkU-8JZnVEd57SPrbvy0S0WMQeupzy79Icvcx_TAgoCYhrrwX6Cxz8DmAkvWEOt85JVhOto2QSlMcQH3UOVhcdaB2jDMkkNRBENfnfV-t46bs6NRge77yD15qBHf-cXLCMeLG2HAaN9DQZM2S4GwRC5yAOYfH7Vc-txiY8dSwvLzYwZNUYvXBY-o7OVHbBQfJiS1bAsCmSKaec'
    assert crypto_service.encryptedSymmetricyKey.enc == 'A256GCM'

    crypto_service.symmetricKey = json_loads(crypto_service.decrypt(crypto_service.sessionPrivateKey['encodedMuk'], crypto_service.encryptedSymmetricyKey))
    assert crypto_service.symmetricKey['k'] == 'w5FzV9t--KgxPDFN2teXJcZ0ekxiTrjmcNlmKmiGZwI'
    assert crypto_service.symmetricKey['alg'] == 'A256GCM'

    crypto_service.encryptedAccountKey = Cipher(crypto_service._get_encrypted_account_key())
    assert crypto_service.encryptedAccountKey.enc == 'A256GCM'
    assert crypto_service.encryptedAccountKey.iv == '09d2f0xTBacV3djN'
    assert crypto_service.encryptedAccountKey.data == 'bgbmVTGji76riFuvhwHq2JNCrJLE246cFX1eZu_pmcliuX1cx-k9hfbXmn4CEIvlI1SWCJtMncEVMH7mxlD7WZBlEqA1kZYZjGfZ-cHUhOW3zxYVDRpVgWyxE-EC79bG0y4W7zb_GH7ucQctfBV1acofio9s_LbiVDFkFd7O1uofrTG3_G-R2GZlnxftC69A4DYiCOpjVukm3ylHt2aD8-hT95HR5NOt2LyDaRRlZA62lFz5XDe9E3eIkK-Kv5qJjmT9ygh40YkMbjwHncPzgXpHxiq08tISP_nL7M918o52Ev9fBo2l3HiExpkQNvUIzQvbpoWB5qQL2rYQcdHKIWFPHuDnpPv8j890uWgvxVCqEUqmpCsIWjbZO9r4wYpwp8ru1ChJpptzpf8V2A8-dBMzmP3QWt7iUsx1PMrQA7I4jb_VbbIIKsuuAKWJv32slBx90PfDoNg8A3GAKfP4kF2YH16TNoNR1GjYnYsx-4YKfQEUrVmnkELtBt51a9eT5GEnbPz2iG4mGOuAHyxtmH9VRqymPG2ivrymaptY4z14qX0TpW7zPtUoV_ej0rzPZwtfEtK8pkS-BQWnBlzobWx6cLubSRnkqLd5hykbsgBgHN33_Xty8xGmHgC2xn3HGiDZsQ'

    crypto_service.set_main_crypto_service()
    assert crypto_service.mainCryptoService.sessionKey == common_data('session_key')
    assert type(crypto_service.mainCryptoService.encryptedSessionPrivateKey).__name__ == 'Cipher'
    assert crypto_service.mainCryptoService.encryptedSessionPrivateKey.iv == 'o86o5GT5zK7xh83e'
    assert crypto_service.mainCryptoService.encryptedSessionPrivateKey.data == 'Fn3jMEhlgsniiwP0wgSUF-HUR2LHEQuIW032Oz25VokclJG9t2Tl8a8KgDHgVwgypO2hQbS_0u2xteDEON4GVV7vf4C_7UmzbWIaz5qN1PXa5_AlnmBmHiRFXoTNEQgqJqxkEDHGnCaNExkw8FenArevjwKwp2Zwr1jIL0m8YVEnROSKg8WEuuOh74EehpkmypBJ-I4PIHe-C80OwqriwK6XK_3dEat6iYzvSYKQK4MwY5eTDSZWoIvaAFs8QHBez8CiaDmzIp1fgNOiYwplf9skc9ZA0cvUc9i0AT3aQa7PBsTHEQ2srgF43sS4eFF6X66QOmsbY4UAy-JjakOmIPkthkkx5dsGIsCUO1whAzIlmnTrNYUFtA'
    assert crypto_service.mainCryptoService.encryptedSessionPrivateKey.enc == 'A256GCM'
    assert crypto_service.mainCryptoService.sessionPrivateKey['encodedMuk'] == '2Zqlkn-ppcrz0RaH3wDUiKwu1YUPj1bRM09R9MEmsrE'
    assert crypto_service.mainCryptoService.sessionPrivateKey['encodedKey'] == 'JH84cS3eNebV8P0Kck3fviRSfnOk8WMFUtiF6umWHFY'
    assert type(crypto_service.mainCryptoService.encryptedSymmetricyKey).__name__ == 'Cipher'
    assert crypto_service.mainCryptoService.encryptedSymmetricyKey.iv == 'uXAcNJaoM54-8r-M'
    assert crypto_service.mainCryptoService.encryptedSymmetricyKey.data == 'Kb69e8qtY7cPkqoG7jMjy1KfG0Il-R5FfsvrLYVJADaan6ICh4f_AUWbdD6aiRe7UvXbqwnaRX-KGfPoCBIrAbQj2ZZsYO_8jcd0RAaYLpnUMLjSgOUh5dnCiksJcDOVvwVDed6xp8LXirt631ArEUc545edlCJi4RohkplHRFdwxWa-6M1al0z0ZM6ac7koVGYYM913CYfsNp3GV7_4ccGg9AQNOJVM6vLTRKg'
    assert crypto_service.mainCryptoService.encryptedSymmetricyKey.enc == 'A256GCM'
    assert crypto_service.mainCryptoService.symmetricKey['k'] == 'DTCXfnskB1Sm8QiMA_qCWxwZ4GmWOeyAreSN8c0pRcc'
    assert crypto_service.mainCryptoService.symmetricKey['alg'] == 'A256GCM'
    assert crypto_service.mainCryptoService.encryptedAccountKey.enc == 'A256GCM'
    assert crypto_service.mainCryptoService.encryptedAccountKey.iv == 'Wfvydy9a5MebX9pk'
    assert crypto_service.mainCryptoService.encryptedAccountKey.data == 'rnU-q-u8Pgb5ZKelOpEZT0HbkjlUCRJ9NqgABLaXhjeXiK1EHOOOYO-l14XC2xr7YBDwQTmG9-kj10KI7yJenOwaR0YIYd-yBkS88_5YilTvQdObGYmnZ6N5DRNZQKQO6ZEY5nJ22QMpMAk5Rt5uwQrB0YXf9MmdJrbMrn0huig-T9GPX6_VwOwYz4WTheq-9QQZJcJRTMszhij0gnPTlKjjagn7ccbdDIJcQsup0AYwhagELZXVhMq6xFh-Sy-6G7dh4JXVWlD5BO2CQv98rD58eDLOjUJlsPsCyMt9X2IwHjaiiQ91kIU9XR5b9MD6v2tPAAOn-5Q7LoyjUXSrCqnT02bvIRGymhO5y4BU-pE7jD76GzPaZ9Lw8MzNOwbXObNR3kadIveaVWxxcf-d2UTmImI3zMoTNvpDlsG17Gl88y2hH1HNHDPZ8K8vPS3GV-N8Y1iv7UYvANigXYYN3iUu2SmtQN0JXfD2lQBlcxhX-4Jzoz1hIlsQ1IpAZ_uMrGYoWOMBJJ69Z3gO6mbLGDk_-fpZwTp82jsvEXverx2Ojan3Q3X80_WX67XSpa5ZyVRl0ipWKkaj7ERjgYo0JEdKni-HCHBdr4M0m1GgtivPGVeRg3ce0Pts7QcWRog'
    assert crypto_service.mainCryptoService.accountKey['email'] == 'dev+onepassword-local-search@dkod.fr'
    assert crypto_service.mainCryptoService.accountKey['masterUnlockKey']['k'] == '2Zqlkn-ppcrz0RaH3wDUiKwu1YUPj1bRM09R9MEmsrE'
    assert crypto_service.mainCryptoService.encryptedPrivateKey.enc == 'A256GCM'
    assert crypto_service.mainCryptoService.encryptedPrivateKey.iv == 'PKQJjGidIGatDZlC'
    assert crypto_service.mainCryptoService.encryptedPrivateKey.data == 'Co--8uAZTsy4texj_Tdyli2ENnzJVFT3vplQfQU96-Jn5GpGXiveljeG3ZbZb_snW93EoM8UIFBupMKvs67wIXbL1P8oU6FdK13ypHXkX1DMyIzvg9NXral3ImIS-0MrmzoH_KRLI-CvDXQ3bf2DXYTAA_x2Dh693rnj4Te3h5GZlM0iI-ziFbVnXSuY71QPz8c44OkjcsJx7EPQJyZuryXqwweObnG2sjA449BeUjtDR1tYId8L82oTlWFjvzWsbyOkWoLl-Dsg6Cax7CMoI4QGUVXCCzXRYWia672IhnhD0WziAi_p4abXn8Rb2hd2YwXnY4PFPWA5Esn8nn-046Ga31ibmF7LraFFV5o9kXP6MWcNjHhXjpQHeJ9ptLfl0eLMuJXnMT9Gwxwp71z5_CQaV92AU6Xg686xz5-b1ZUT7ksV6z3UqNK-i8eeOwRJ5ARJwzQRBOyR-A9oYhPwiH_flxEbWzW15hPje5zDMbT5mKQXLRB3hLPltgOcqQsBtY-3KHBWUj2y0ZueBASBYiHnWoAEkIf9u0-gAMVumf4ZHf03EcXm98qV4VCSlDM1uO1W3gNxyg8f3B8F4fmazDxYTVe4kUSVLE5qUqCGxVp-uXS1dgaDwb0T40MDtgnENHMerbSdLg7f7CdRiZtGEJk0ahU6W9LFuAkFxenEALE6-BymFMD_-Z3OYV_ivbQVaC7nwhrDMWfndlcDuZbDpRSwS75iZWs9UiWOrdA3AS6tjWVb9_n_glEYoPG6Nflf_yuAAaZGBI57DsYVSyW_OjNf_5YyYfbAW0Yaptk7r65S5DJB4cKFkbQQtFdHUONe3nQnxexXmirpbIfF97j0nRu8JdqVZsY3X1KcA-abNIq8T0fSi_GrwyMq9qa6JqNRYyrzA5Wau7sT_eSBSu2Ls9UvVRRjISK3oVBbJGhKH5nuHldYsqkGsmyfXKogkjF7PZAS4AgwGUmgLiPtsZvnjsSQdC3K8cyhNYDfmFd5_C3xgxoBEjnpwMTH5_mIo6n_0lwfM7hp-KkQiDnG03bBhauhlw5Af1BfMnFUk6mbEoX3ndSFZ3iF2ckW4QVL-unLKb0eUnjD_9dP6lht7gfgSUSaZoo9QLmLgwD1zQ4iV1bd1R8DRcpUio9lakR5lbjVdRPUcPiwkrmmdtzo1Ve0XM8TLHlo3Bfq8Dh6HYV6ANl4K25GRKCcWjT0n7_-D1qYQSeF2tZDs4IBPD_NbVNmbE_-8jK03n367E87wJ6Vw-wKcUjlB8Q2rMcqGSmp1OQMq1L5EtrXDn_WiOd2JDXaVbo1FyveNC5KVTJGFYDbDzaBkjZFFMzc_ct-uxaiMMrDEiHmwQwvUS1mX5ooi6altsNsxXs855iTAMrDC8wtZYQX8EYkskTLc9sDKYpEUNu8sqxfyd4WtW5F9Pz6KdzDaxmcnma6_SPVTnOnV7pPcyFbG8dF00YwlSPN8RoNgjnqzVoXyNTilj9mRpsnMzonME_sH3_kJ4KdUQfYyEhbtbZLRXnj1VsufB-7p22_B69qy2sKSZFuKK7pbyhHtV2fpSYQtkcgOy1RKt6YbnBc5aV3QZwtAFjVVLAtZPGLt_0J_z3wQJYRGB4q6zBnqWMKipkR-9xEkEKyaG5jy8Y7R87JtPLs_M3vRQ3gvAQmWlYEYmgAJ1wTpc-1cjZ7xgQ8T8ZVGEWYOwH9P9H1ujDsAJkokbedwWjZJbCyZTHmEBs_afaebde0nlrvcNm2CGn7kYJs9VKmME5zoryDaEx934ALKUCe7dSOqvj_498AwYW3Co_H-JPBk19EEKckYmmD86n170UNO8dZMSkqrQufRTT-aRhAVWKXc5yrK68K7MqHVLCqQaxOT78CIRbxKwZbnGLuvdpT6zgp9OJAJPV3nFx_sLWJ1-FhU5sB2Sbiy3d4CdgknXa3HolhpPGnisnqpK1IB5L5VcWHH-uzyN2JY71eiEqD7j4Pvr1flA8QvMPXpmZQl44ri3gkZGi6KpsgIJcatWPi-UfoQuXFWyOa0Kj1SbCMOFURZPZiHZQhaKNpNsQd8m7fxXZGPUAAenfyVp1APqJtk_U8iz92ib_cKqL-soFZSSZBJfV4PKbhHWat5hPwqlH7nrz6iIWc0VGXO-RTeciAWgptU30loMy10DoFj5xxAwCj-_L-gu5oO35p1W3noNJ_sF_2zgxBXIl0TRmwKInQYHdWuTPvdGMsE4p9Bsp_I7UJ7ox_yf0aq7KxMutAg6uaJIwnn5zKnfXlpu9qKUwjm7a8PVMk2MQnH8KRCMwV'
    assert crypto_service.mainCryptoService.privateKey['alg'] == 'RSA-OAEP'
    assert crypto_service.mainCryptoService.privateKey['d'] == 'AmCxuabUgIyblLBolHSHvZEh_b7PzswAJquPazw9hu6EaN__noNHFIYMYLovfZ99B58XxiuBSo-N7FPMpWGtVTKPBkR1leSp2bxtnz_M25wRquUxJ3BSWy6dNs5pahRu27irQ1b5cTNRrsCm_ew0aviC_7YgbauXgBK3SBmFXRH81Cw_5XCXmO-9Y7TIPwAnd2jThkyyiyZ9KwSyD96h8eqkn6LX_dXYRRnzlCcTzg2TYXWvtynl1S6z4vRmUKmYxmRroFrIukto5Wlq6o0F8nQ_KxvMrud8qMkatuKcaE9LpAzqXHnCsXf7E_n4UKgUCtLkTrQFIqRiJVSs7R_rQQ'

    crypto_service.accountKey = json_loads(crypto_service.decrypt(crypto_service.mainCryptoService.symmetricKey['k'], crypto_service.encryptedAccountKey))
    assert crypto_service.accountKey['email'] == 'dev+onepassword-local-search-user@dkod.fr'
    assert crypto_service.accountKey['masterUnlockKey']['k'] == 'j4mKCHJyeG-O187LjWvjfYciZnKWrDZ8MjS6jhBNGnc'

    crypto_service.encryptedPrivateKey = Cipher(crypto_service._get_encrypted_private_key())
    assert crypto_service.encryptedPrivateKey.enc == 'A256GCM'
    assert crypto_service.encryptedPrivateKey.iv == 'ZLveVKZLQPv-KB6T'
    assert crypto_service.encryptedPrivateKey.data == 'nZMdB0iqEtPng_bOYBy-Ubda34qZSBRXvyDw98TxFAeK9Ol2xcqVcScP2aWjwU9YytsrHJSzFyv_DBwCD81LpxM__a55XjtJoJg_wn6FqGn58fvOwfTToT21rrByqIQswd4RjWIEpekCukMQfXiI1z0AA2Dz8kFJM4NvBzW-QDrxad4yveKEXKcEnuNeVTqUdCjlEp1CtVMQSc6nGFeSTBc9JGOXxTtng2nMpArkifZg9daImkgDvxLbm2O1NyVqLh-IU8YKX2y83LBIUAanXDoeaqb1mOgQUPpoa3Qicp7SRW_eTTtMrujt89NLlhGBklesS1s_kKTPUcF_eBaMccZSZ6Fn7isYMAojdQ3lN98PJ9-OMknA58fImWYV-T6wzZ5PGPq5gsRvktBYqKqo5x8gAFAb1F6wynT3PVFYdQZTLVsdQbiDhVIGfX0nX-F6XbKpa7tbSY2NtMvScIW1R-J6ZIA9_FWQteWQ-PLUXqFAiAFBa0W_TGSklUPZo4BGwMvKAgNEMtmCztbStRASx4suAwdz-5XQh2DuuXZk-7WIZnJf1bVi8FeylqXkVlDtHA56wO9Nul1RUfCuyIFrxJEwjX6eMaX6ovJDa2nmR7keY6pyGEjHSGWedZO3C2I05FHYCCH3Z0bVGHAnOIrlZFpluxtGzl2ods7kw_c30m9FKWKZuMDBrwoLNnIWlEd-XWJEeOZxgwzyt71gGB5tz7DrxF4jIxqZIKiQ_ecK9H7OZqUiuWxgCrMgqeaGk6fDlicM2aNsFpdHAkNrtBrRCSsgiamS0p3BkkdQUZ4QHoommA7sU5jsf2fp-8yIoMj8b1XzEQBxu4ID86VsVwZsbV8FGm8UCe1mQhVWr_bZOaUrufT4CdQk7_TXvgxC-mt2EHB-bfh6YcxSx-0OFQPH5ymrUdAULu9v3i7hdZuhXf-XzPPmHf-z1YHmNzckIwvJkX9JqpUhL0O5VsWMWqXp23c7UZIJQpPUJBz_BkYP304nx4T_8GUN5c2ukJa0EiLsgERd6vcYGVPNp9qoS-c0GaCDYYe7RclVCyu6Z4CmwV35EM_sW-e6Cu3qvKkNVrlekPgbAtMyeC5vYhDubN0wnZZLXOKqjk9eaAkUoNBsHSifLQUG6nPPslOj4c_jMMBb9c04-ZR-3I8VvYILuTG34shf34JqILQsgSskm7HGiHnZhOvY_bqlS_gvIMHKbbSWk0-HeHLghlI6oJTvhGkkjoHPbFPy5wzSGb7hXdoqza4ZFz26Hjme12KcnvczqVn4rRHiOVdAmvGllMmkjs_g64ZxpIcd2BiY8nKreHVBMGbGkAvxd2o4a2a05OAZBpyAQQq2TeZdheQXZCef-9D5Oamq08vnjEqqe8HS8arEY_U9KoZyIdblrrVMfOhFvC9qAvMstsaQTG5LhEk_ZDlYICJWVmsLXmySFngZvUQj0if4R8K3WRzxzRPrNIZJJQxzr26YQWrIXG6KKpQ3q_PjHc1Q7wvXAT4r8jN_-quhY39W2mQS9DVN9AfXXMT46nL0Vt-A7Ao4htPzg0c555yWo-4MAyPAsCnasEIbf762JYBeWM7DYDmGBQgDXd7OzGvYv90Wi_eCvcELx9U86M631rHpvwbYhCQASNbOWRH21Ows1ACvohGWZTVs3MWv77f92vLnTj_KWBgqIsmUfn9W1MwmA2wB47TSnK62FR-nq7K2_Lc-yY2xy6wfVZbWnhm-F6tQryMg295GQ2ohfia1j61Is_toH6Sh-0motUVYAwfOxaHmImBHfZDNsPwIgkqRKK1MdmDSR_wxeP92DD7TexmH4oc5Q1p3OFgiePAeN3OG9g0yOjACSWkKBSTjhyuWE9ol-MMM53d7RKcuNedV3G9fGKqP7ry3Tqm0cFF6iHNcThISGcQ909hWT_CoLuq9Qn4xEoQiVPgKftiMXCABQdy7Ehq3h33kan-Y8daVZdiSi32P3a0wQJGPLs0FjnHo351ul7YVa1_V3QUbhQRwqIhsAvdn4leUAzuz9Ko7ac4Wq2RHXyGouoGZGHpaDKSo5CB5XuNgg5p7pAPgib_111_-PT8XvbTedPT7VqajTqaI6gL92o8_EAjf-aXNZhZdQ6RsQoLDDRL3ppUrYRo5VcpQzysNLdezYFQB2-Ucw98B2Mzp5Q-iMnFjE6PC8y_yykHhHSEIvxmk9QMsxBaTSWso3Sj47-OsOsCzirrP6ZaHLlVERLeEmae06b0GCbBjKW3YgGbja8CBJ_xPUoLDNq3hW0jFiLAM7Kl-j7JoDmGz-147'

    crypto_service.privateKey = json_loads(crypto_service.decrypt(crypto_service.symmetricKey['k'], crypto_service.encryptedPrivateKey).decode('utf-8'))
    assert crypto_service.privateKey['alg'] == 'RSA-OAEP'
    assert crypto_service.privateKey['d'] == 'XGIP8FQ8oFNft9glFdmkTvJ0rcMuAQrnM3h1aFLPx-8EL8g196VkQEb9nS2yhE7hUgSkqe50aa3DQSBq_1mcqeHYhh6rhPXfIrIs8YHmsybC5XhPQabYnaNxTyANZBkSRtlM9OY6QzZHZYtmg1XpMgJ3uSzUmHA5CSVWb88x7KY5DadCE7Qdas7ctTCgShg4chJUMrvZSP961r4g7sCL83vLiVy0RPeuK9Zd72QHDtvSuDxwAzJk1McmAbsxQsihfNesIicc04ZSf6oKPGxhqT7zU1t1cZ9FB5gutNyTfBCt-ldyNd425mkGJcvGQiYRJDvF7MN41ZfPKy-GysLQTQ'




from onepassword_local_search.models.Cipher import Cipher
from json import dumps as json_dumps
from onepassword_local_search.exceptions.ManagedException import ManagedException


class Item:

    id: str
    uuid: str
    vault_id: str
    encryptedOverview: Cipher
    encryptedDetails: Cipher
    overview: dict
    details: dict

    def __init__(self, row):
        self.id = row['id']
        self.uuid = row['uuid']
        self.vaultId = row['vault_id']
        self.encryptedOverview = Cipher(row['overview'])
        self.encryptedDetails = Cipher(row['details'])

    def _search_recursive_in_sections(self, field):
        # Fallback: search fieldName is all sections
        for section in self.details.get('sections'):
            if section.get('fields'):
                for f in section['fields']:
                    if (f['t'] == field or f['n'] == field) and 'v' in f.keys():
                        return f['v']
        return None

    def _search_recursive_in_section(self, section_name, field):
        for section in self.details.get('sections'):
            if section['title'] == section_name:
                for f in section['fields']:
                    if (f['t'] == field or f['n'] == field) and 'v' in f.keys():
                        return f['v']
        return None

    def get(self, field=None, strict=True, output=True):
        if field is None:
            self.__delattr__('encryptedOverview')
            self.__delattr__('encryptedDetails')
            return self.output(self, output)

        path = field.split(';;')
        out = None

        if field == 'totp':
          out = self.get_totp()
        elif field in ['title', 'url']:
            out = self.overview.get(field)
        elif field in ['username', 'password'] and (out is None or out == ''):
            out = self.details.get(field)
            if out is None:
                out = self._get_details_field(field)
        elif field == 'notesPlain' and (out is None or out == ''):
            out = self.details.get(field)
        elif hasattr(self, field) and (out is None or out == ''):
            out = self.__getattribute__(field)
        elif len(path) > 1 and (out is None or out == ''):
            out = self._search_recursive_in_section(path[0], path[1])
        if out is None and self.details.get('sections'):
            out = self._search_recursive_in_sections(field)

        if strict and out is None:
            raise ManagedException('Unable to find field %s of item %s ' % (field, self.uuid))
        elif not strict and out is None:
            return ''

        return self.output(out)

    def _get_details_field(self, field):
        if self.details.get('fields'):
            for f in self.details.get('fields'):
                if f['name'] == field:
                    return f['value']
        return None

    def output(self, content, output=True):
        if not output:
            return content
        out_type = type(content).__name__
        if out_type in ['str', 'int']:
            return content
        else:
            return json_dumps(content.__dict__)

    def get_totp(self):
        import pyotp
        token = self._search_recursive_in_sections('One-time password')
        if token:
            return pyotp.TOTP(token.replace(' ', '')).now()
        else:
            raise Exception('Item %s doesn\'t seem to have a field untitled "One-time password"' % self.uuid)




from onepassword_local_search.services.StorageService import StorageService
from onepassword_local_search.services.ConfigFileService import ConfigFileService
from onepassword_local_search.services.CryptoService import CryptoService
from onepassword_local_search.services.SecondaryCryptoService import SecondaryCryptoService
from sys import stderr
from os import environ as os_environ


class AccountService:

    accounts: []
    existing_accounts: []
    storageService: StorageService
    configFileService: ConfigFileService
    cryptoServices: {} = {}

    def __init__(self, storage_service: StorageService, config_file_service: ConfigFileService):
        self.storageService = storage_service
        self.configFileService = config_file_service
        self.existing_accounts = self.configFileService.get_accounts()
        self.accounts = self.get_available_accounts()
        self.available_vaults = self.get_available_vaults()
        self.available_vaults_id = self.get_available_vaults_id()

    def get_available_accounts(self):
        accounts = []
        for account in self.existing_accounts:
            if not os_environ.get('OP_SESSION_' + account['shorthand']):
                continue
            account_id = self.storageService.get_account_id_from_user_uuid(account['userUUID'])
            if account_id:
                account['id'] = account_id
                accounts.append(account)
        return accounts

    def get_available_accounts_id(self):
        return [str(account['id']) for account in self.accounts]

    def get_available_vaults(self):
        return self.storageService.get_vaults_owned_by_accounts(self.get_available_accounts_id())

    def get_available_vaults_id(self):
        if self.available_vaults and len(self.available_vaults) > 0:
            return [vault['id'] for vault in self.available_vaults]
        return []

    def set_crypto_services(self):
        services = {}
        not_logged_accounts = []
        try:
            for account in self.accounts:
                if self.is_main_account(account):
                    crypto_class = CryptoService
                else:
                    crypto_class = SecondaryCryptoService
                try:
                    services[account['id']] = crypto_class(self.storageService, self.configFileService, account['id'])
                except Exception:
                    not_logged_accounts.append(account)
                    pass
            if len(not_logged_accounts) > 0:
                raise Exception('Not authenticated')
        except Exception:
            for account in not_logged_accounts:
                if account['shorthand'] == 'my':
                    over = 'your personal account'
                else:
                    over = 'the team %s' % account['shorthand']
                print('You are not authenticated over "%s"' % over, file=stderr)
            raise
        return services

    def get_main_crypto_service(self):
        return self.cryptoServices['1']

    def get_decryptor(self, vaultId):
        if self.cryptoServices == {}:
            self.cryptoServices = self.set_crypto_services()
        for vault in self.available_vaults:
            if vaultId == vault['id']:
                account_id = vault['account_id']
                break
        else:
            raise Exception('Unable to find proper decryptor for vault %s' % vaultId)

        if account_id in self.cryptoServices.keys():
            return self.cryptoServices[account_id]
        else:
            raise Exception('Unable to find proper decryptor for vault %s' % vaultId)

    def is_main_account(self, account):
        return account['id'] == 1


from onepassword_local_search.services.StorageService import StorageService
from onepassword_local_search.services.CryptoService import CryptoService
from onepassword_local_search.models.Item import Item
from onepassword_local_search.services.ConfigFileService import ConfigFileService
from onepassword_local_search.services.AccountService import AccountService
from onepassword_local_search.exceptions.ManagedException import ManagedException
import string


class OnePassword:

    storageService: StorageService
    cryptoService: CryptoService
    configFileService: ConfigFileService
    accountService: AccountService

    def __init__(self, custom_uuid_mapping=None):
        self.storageService = StorageService(custom_uuid_mapping)
        self.configFileService = ConfigFileService()
        self.accountService = AccountService(self.storageService, self.configFileService)
        if custom_uuid_mapping:
            self._ensure_uuid_mapping_has_values()

    def _ensure_uuid_mapping_has_values(self):
        if not self.storageService.uuid_mapping_has_entries():
            self.mapping_update()

    def can_decrypt(self, item):
        return item.vaultId in self.accountService.available_vaults_id

    def get(self, uuid, field=None, custom_mapping=None, output=True):
        encrypted_item = Item(self.storageService.get_item_by_uuid(uuid, custom_mapping))
        if not self.can_decrypt(encrypted_item):
            print('You are not connected to the required account to decrypt item: %s' % uuid)
            exit(1)
        decryptor = self.accountService.get_decryptor(encrypted_item.vaultId)
        item = decryptor.decrypt_item(encrypted_item)
        decrypted_field = item.get(field, output=output)
        if output:
            print(decrypted_field, end='')
        return decrypted_field

    def get_items(self, result_fitler):
        items = []
        for item in self.storageService.list(self.accountService.get_available_accounts_id()):
            encrypted_item = Item(item)
            decryptor = self.accountService.get_decryptor(encrypted_item.vaultId)
            decrypted_item = decryptor.decrypt_item(encrypted_item)
            if result_fitler is None or result_fitler in decrypted_item.overview['title']:
                items.append(decrypted_item)
        return items

    def list(self, result_format=None, result_fitler=None):
        class SimpleFormatter(string.Formatter):
            def get_value(self, key, args, kwargs):
                return item.get(key, strict=False)
        list_format = result_format if result_format else '{uuid} {title}'
        sf = SimpleFormatter()
        for item in self.get_items(result_fitler):
            print(sf.format(list_format, item).strip())

    def mapping(self, subcommand, use_lastpass_uuid=False):
        self.storageService.checks_for_uuid_mapping()
        if subcommand == 'list':
            self.mapping_list(use_lastpass_uuid)
        else:
            return getattr(self, 'mapping_' + subcommand)()

    def mapping_update(self):
        self.storageService.truncate_uuid_mapping_table()
        for item in self.storageService.list(self.accountService.get_available_accounts_id()):
            encrypted_item = Item(item)
            decryptor = self.accountService.get_decryptor(encrypted_item.vaultId)
            decrypted_item = decryptor.decrypt_item(encrypted_item)
            custom_uuid = decrypted_item.get('UUID', strict=False)
            lastpass_uuid = decrypted_item.get('LASTPASS_ID', strict=False)
            if custom_uuid:
                self.storageService.add_uuid_mapping(custom_uuid=custom_uuid, op_uuid=decrypted_item.uuid,
                                                     lpass_uuid=lastpass_uuid)
        self.storageService.con.commit()

    def mapping_list(self, use_lastpass_uuid=False):
        mapping_field_to_use = 'lpass_uuid' if use_lastpass_uuid else 'custom_uuid'
        for item in self.storageService.list_mapping():
            print("%s <-> %s" % (item['op_uuid'], item[mapping_field_to_use]))

    @staticmethod
    def version():
        from .__version__ import __version__
        print('Version: ' + __version__)

    def is_authenticated(self, user_uuids=None):
        try:
            if self.accountService.cryptoServices == {}:
                self.accountService.cryptoServices = self.accountService.set_crypto_services()
        except Exception:
            pass
        if user_uuids is None:
            user_uuids = [self.configFileService.get_user_uuid_from_latest_signin()]
        if isinstance(user_uuids, str):
            user_uuids = [user_uuids]
        for user_uuid in user_uuids:
            account_id = self.storageService.get_account_id_from_user_uuid(user_uuid)
            if account_id in self.accountService.cryptoServices.keys():
                try:
                    if not self.accountService.cryptoServices[account_id].is_authenticated():
                        return False
                except:
                    return False
            else:
                return False
        return True



import pytest
from onepassword_local_search.services.ConfigFileService import ConfigFileService
from onepassword_local_search.services.AccountService import AccountService
from onepassword_local_search.services.StorageService import StorageService
from onepassword_local_search.tests.fixtures_common import op_session, op_personal_session, op_dual_session


@pytest.mark.usefixtures("op_session")
def test_available_accounts_team_only():
    accounts = AccountService(StorageService(), ConfigFileService()).accounts
    assert len(accounts) == 1
    assert accounts[0]['shorthand'] == 'onepassword_local_search'


@pytest.mark.usefixtures("op_dual_session")
def test_available_accounts_dual_session():
    accounts = AccountService(StorageService(), ConfigFileService()).accounts
    assert len(accounts) == 2
    shorthands = sorted([ accounts[0]['shorthand'], accounts[1]['shorthand']])
    assert shorthands == ['my', 'onepassword_local_search']


@pytest.mark.usefixtures("op_dual_session")
def test_available_vaults_dual_session():
    vaults = AccountService(StorageService(), ConfigFileService()).available_vaults
    assert len(vaults) == 4
    assert sorted([vault['uuid'] for vault in vaults]) == ['jih4mxjnvmpcollannoie5gt4u', 'rloeunbmcvvniz7wedijvqhnhu', 'wbt4wkbfztldfolbdfyo2rnfdu', 'zcud5s5loxyjcaig545zwkxe2i']


@pytest.mark.usefixtures("op_session")
def test_available_vaults_main_session():
    vaults = AccountService(StorageService(), ConfigFileService()).available_vaults
    assert len(vaults) == 3
    assert sorted([vault['uuid'] for vault in vaults]) == ['jih4mxjnvmpcollannoie5gt4u', 'rloeunbmcvvniz7wedijvqhnhu', 'wbt4wkbfztldfolbdfyo2rnfdu']



from os import environ
from sys import excepthook, stderr


def exception_handler(exception_type, exception, traceback):
    if environ.get('DEBUG'):
        excepthook(exception_type, exception, traceback)
    else:
        print("%s" % exception, file=stderr)



import pytest
import os
import json
from pytest_mock import mocker
from onepassword_local_search.services.ConfigFileService import ConfigFileService


def test_without_env_var(mocker):
    mocker.patch.object(ConfigFileService, '_get_local_config')
    with open(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'tests', 'config'), 'r') as f:
        config = f.read()
        f.close()
    ConfigFileService._get_local_config.return_value = json.loads(config)
    config_file_service = ConfigFileService()
    assert config_file_service.latest_signin == 'onepassword_local_search'


def test_with_env_var(monkeypatch):
    monkeypatch.setenv('ONEPASSWORD_CONFIG_FILE_PATH', os.path.join(os.path.dirname(os.path.dirname(__file__)), 'tests', 'config'))
    config_file_service = ConfigFileService()
    assert config_file_service.latest_signin == 'onepassword_local_search'


from onepassword_local_search.services.StorageService import StorageService
from onepassword_local_search.services.ConfigFileService import ConfigFileService
from onepassword_local_search.models.Cipher import Cipher
from onepassword_local_search.lib.optestlib import aes_decrypt, get_binary_from_string, rsa_decrypt, determine_session_file_path_from_session_key
from os import environ as os_environ, path as os_path
from json import loads as json_loads
from glob import glob as glob_glob
import sys
from onepassword_local_search.services.CryptoService import CryptoService


class SecondaryCryptoService(CryptoService):

    mainCryptoService: CryptoService

    def __init__(self, storage_service: StorageService, config_file_service: ConfigFileService, account_id):
        super().__init__(storage_service, config_file_service, account_id)
        self.set_main_crypto_service()

    def _get_base_keys(self):
        self.sessionKey = self._get_session_key()
        self.encryptedSessionPrivateKey = self._get_encrypted_session_key()
        self.sessionPrivateKey = json_loads(self.decrypt(self.sessionKey, self.encryptedSessionPrivateKey))
        self.encyptedSymmetricyKey = Cipher(self._get_encrypted_symmetric_key())
        self.symmetricKey = json_loads(self.decrypt(self.sessionPrivateKey['encodedMuk'], self.encyptedSymmetricyKey))
        self.encryptedAccountKey = Cipher(self._get_encrypted_account_key())
        self.accountKey = json_loads(self.decrypt(self.mainCryptoService.symmetricKey['k'], self.encryptedAccountKey))
        self.encryptedPrivateKey = Cipher(self._get_encrypted_private_key())
        self.privateKeyRaw = self.decrypt(self.symmetricKey['k'], self.encryptedPrivateKey).decode('utf-8')
        self.privateKey = json_loads(self.privateKeyRaw)

    def set_main_crypto_service(self):
        #TODO: singleton
        self.mainCryptoService = CryptoService(self.storageService, self.configFileService, self.storageService.get_account_id_from_user_uuid(self.get_main_user_uuid()))
        self.mainCryptoService._get_base_keys()

    def get_main_user_uuid(self):
        return self.storageService.get_main_user_uuid()



from onepassword_local_search.services.StorageService import StorageService
from onepassword_local_search.services.ConfigFileService import ConfigFileService
from onepassword_local_search.models.Cipher import Cipher
from onepassword_local_search.lib.optestlib import aes_decrypt, get_binary_from_string, rsa_decrypt, determine_session_file_path_from_session_key
from os import environ as os_environ, path as os_path
from json import loads as json_loads
from glob import glob as glob_glob
import sys


class CryptoService:

    accountId: int
    storageService: StorageService
    sessionKey: str
    encryptedSessionPrivateKey: Cipher
    encryptedSymmetricyKey: Cipher
    encryptedAccountKey: Cipher
    encryptedPrivateKey: Cipher
    configFileService: ConfigFileService
    accountKey: dict
    symmetricKey: dict
    privateKey: dict
    privateKeyRaw: str
    shorthand: str
    userUUID: str
    vaultKeys: dict = {}

    def __init__(self, storage_service: StorageService, config_file_service: ConfigFileService, account_id):
        self.storageService = storage_service
        self.configFileService = config_file_service
        self.userUUID = self.storageService.get_user_uuid_from_account_id(account_id)
        self.shorthand = self.configFileService.get_shorthand_from_user_uuid(self.userUUID)

    def _get_base_keys(self):
        self.sessionKey = self._get_session_key()
        self.encryptedSessionPrivateKey = self._get_encrypted_session_key()
        self.sessionPrivateKey = json_loads(self.decrypt(self.sessionKey, self.encryptedSessionPrivateKey))
        self.encryptedSymmetricyKey = Cipher(self._get_encrypted_symmetric_key())
        self.symmetricKey = json_loads(self.decrypt(self.sessionPrivateKey['encodedMuk'], self.encryptedSymmetricyKey))
        self.encryptedAccountKey = Cipher(self._get_encrypted_account_key())
        self.accountKey = json_loads(self.decrypt(self.symmetricKey['k'], self.encryptedAccountKey))
        self.encryptedPrivateKey = Cipher(self._get_encrypted_private_key())
        self.privateKeyRaw = self.decrypt(self.symmetricKey['k'], self.encryptedPrivateKey).decode('utf-8')
        self.privateKey = json_loads(self.privateKeyRaw)

    def _get_session_key(self):
        if not os_environ.get('OP_SESSION_' + self.shorthand):
            print('Environment variable OP_SESSION_team is not set for %s ' % self.shorthand)
            exit(1)
        return os_environ.get('OP_SESSION_' + self.shorthand)

    def _get_encrypted_session_directory_path(self):
        if os_environ.get('OP_SESSION_PRIVATE_KEY_FOLDER'):
            path = os_environ.get('OP_SESSION_PRIVATE_KEY_FOLDER')
            if os_path.isdir(path):
                return path

        # TODO: implement for other platforms than MAC
        path = os_path.join(os_environ.get('TMPDIR'), 'com.agilebits.op.501')
        if not os_path.isdir(path):
            raise Exception('Session private folder is missing. Ensure you are signin with the onepassword cli (op).')
        return path

    def _get_encrypted_session_file_path(self):
        if os_environ.get('OP_SESSION_PRIVATE_KEY_FILE'):
            path = os_environ.get('OP_SESSION_PRIVATE_KEY_FILE')
            if os_path.isfile(path):
                return path
        filepath = os_path.join(self._get_encrypted_session_directory_path(), determine_session_file_path_from_session_key(self.sessionKey))
        if not os_path.isfile(filepath):
            raise Exception('Unable to find session file')
        return filepath

    def _get_encrypted_session_key(self):
        with open(self._get_encrypted_session_file_path()) as f:
            return Cipher(f.read())

    def _get_encrypted_user_symmetric_key(self):
        account_id = self.storageService.get_account_id_from_user_uuid(self.userUUID)
        return self.storageService.get_encrypted_symmetric_key(account_id)

    def _get_encrypted_symmetric_key(self):
        account_id = self.storageService.get_account_id_from_user_uuid(self.userUUID)
        return self.storageService.get_encrypted_symmetric_key(account_id)

    def _get_encrypted_account_key(self):
        account_id = self.storageService.get_account_id_from_user_uuid(self.userUUID)
        return self.storageService.get_account_key(account_id)

    def _get_encrypted_private_key(self):
        account_id = self.storageService.get_account_id_from_user_uuid(self.userUUID)
        return self.storageService.get_encrypted_private_key(account_id)

    def decrypt(self, key, cipher: Cipher):
        return aes_decrypt(
            ct=get_binary_from_string(cipher.data)[:-16],
            key=get_binary_from_string(key),
            iv=get_binary_from_string(cipher.iv),
            tag=get_binary_from_string(cipher.data)[-16:])

    def _get_vault_key(self, vault_id):
        if not hasattr(self, 'privateKeyRaw') or not self.privateKeyRaw:
            self._get_base_keys()
        account_id = self.storageService.get_account_id_from_user_uuid(self.userUUID)
        encrypted_vault_key = json_loads(self.storageService.get_encrypted_vault_key(vault_id, account_id))
        return json_loads(rsa_decrypt(self.privateKeyRaw, encrypted_vault_key['data']).decode('utf-8'))

    def decrypt_item(self, item, part='full'):
        vault_key = None
        if os_environ.get('VAULT_%s_KEY' % item.vaultId):
            vault_key = os_environ.get('VAULT_%s_KEY' % item.vaultId)
        elif not self.vaultKeys.get(item.vaultId):
            self.vaultKeys[item.vaultId] = self._get_vault_key(item.vaultId)
        if not vault_key:
            vault_key = self.vaultKeys[item.vaultId]['k']

        if part == 'overview':
            item.overview = json_loads(self.decrypt(vault_key, item.encryptedOverview).decode('utf-8'))
        elif part == 'details':
            item.details = json_loads(self.decrypt(vault_key, item.encryptedDetails).decode('utf-8'))
        else:
            item.overview = json_loads(self.decrypt(vault_key, item.encryptedOverview).decode('utf-8'))
            item.details = json_loads(self.decrypt(vault_key, item.encryptedDetails).decode('utf-8'))

        return item

    def is_authenticated(self):
        try:
            self._get_base_keys()
            return True
        except:
            pass
            return False



import pytest
from onepassword_local_search.lib.optestlib import determine_session_file_path_from_session_key
from onepassword_local_search.tests.fixtures_common import common_data


def test_determine_session_file_path_from_session_key():
    assert determine_session_file_path_from_session_key(common_data('session_key')) == common_data('session_filename')
    assert determine_session_file_path_from_session_key('kYOPcBsWTNQ81pfLvlkRs0ogDmooRvs3YKWgKmwLGFA') == '.-eqI_7WLuKYKW7eMBaSmJpEuVrQ'
    assert determine_session_file_path_from_session_key('peuf3297Q_hmWr8RFHi-jCskxnC_v1fmqYK8VdPVtPQ') == '.hHvZhhoB807qm-E2BuK04C_Q60Q'
    assert determine_session_file_path_from_session_key('7hwQ4p4S7UnebRB3bpcy2br_ktUGyd0aGEtRob4im3M') == '.eaQClbtT1fMTfuNJzPkVGC2DsZI'
    assert determine_session_file_path_from_session_key('15RvkcSJWi1TSYlRX6jc-J5IAoYFa5_26C-V6ncptHs') == '.gLR-shwH24wVDAB5E_LoDYDkfDw'


import pytest
from onepassword_local_search.services.CryptoService import CryptoService
from onepassword_local_search.services.ConfigFileService import ConfigFileService
from onepassword_local_search.tests.fixtures_common import common_data, op_session, op_personal_session
from onepassword_local_search.services.StorageService import StorageService
from onepassword_local_search.models.Cipher import Cipher
from onepassword_local_search.models.Item import Item
import os
from json import loads as json_loads


@pytest.mark.usefixtures("op_session")
@pytest.fixture
def crypto_service(mocker):
    return CryptoService(StorageService(), ConfigFileService(), 1)


@pytest.mark.usefixtures("op_personal_session")
@pytest.fixture
def crypto_service_personal(mocker):
    return CryptoService(StorageService(), ConfigFileService(), 2)


@pytest.mark.usefixtures("op_session")
def test_session_keys(crypto_service):
    crypto_service.sessionKey = crypto_service._get_session_key()
    assert crypto_service.sessionKey == common_data('session_key')
    crypto_service.encryptedSessionPrivateKey = crypto_service._get_encrypted_session_key()
    assert type(crypto_service.encryptedSessionPrivateKey).__name__ == 'Cipher'
    assert crypto_service.encryptedSessionPrivateKey.iv == 'o86o5GT5zK7xh83e'
    assert crypto_service.encryptedSessionPrivateKey.data == 'Fn3jMEhlgsniiwP0wgSUF-HUR2LHEQuIW032Oz25VokclJG9t2Tl8a8KgDHgVwgypO2hQbS_0u2xteDEON4GVV7vf4C_7UmzbWIaz5qN1PXa5_AlnmBmHiRFXoTNEQgqJqxkEDHGnCaNExkw8FenArevjwKwp2Zwr1jIL0m8YVEnROSKg8WEuuOh74EehpkmypBJ-I4PIHe-C80OwqriwK6XK_3dEat6iYzvSYKQK4MwY5eTDSZWoIvaAFs8QHBez8CiaDmzIp1fgNOiYwplf9skc9ZA0cvUc9i0AT3aQa7PBsTHEQ2srgF43sS4eFF6X66QOmsbY4UAy-JjakOmIPkthkkx5dsGIsCUO1whAzIlmnTrNYUFtA'
    assert crypto_service.encryptedSessionPrivateKey.enc == 'A256GCM'
    crypto_service.sessionPrivateKey = json_loads(crypto_service.decrypt(crypto_service.sessionKey, crypto_service.encryptedSessionPrivateKey))
    assert crypto_service.sessionPrivateKey['encodedMuk'] == '2Zqlkn-ppcrz0RaH3wDUiKwu1YUPj1bRM09R9MEmsrE'
    assert crypto_service.sessionPrivateKey['encodedKey'] == 'JH84cS3eNebV8P0Kck3fviRSfnOk8WMFUtiF6umWHFY'
    crypto_service.encryptedSymmetricyKey = Cipher(crypto_service._get_encrypted_symmetric_key())
    assert type(crypto_service.encryptedSymmetricyKey).__name__ == 'Cipher'
    assert crypto_service.encryptedSymmetricyKey.iv == 'uXAcNJaoM54-8r-M'
    assert crypto_service.encryptedSymmetricyKey.data == 'Kb69e8qtY7cPkqoG7jMjy1KfG0Il-R5FfsvrLYVJADaan6ICh4f_AUWbdD6aiRe7UvXbqwnaRX-KGfPoCBIrAbQj2ZZsYO_8jcd0RAaYLpnUMLjSgOUh5dnCiksJcDOVvwVDed6xp8LXirt631ArEUc545edlCJi4RohkplHRFdwxWa-6M1al0z0ZM6ac7koVGYYM913CYfsNp3GV7_4ccGg9AQNOJVM6vLTRKg'
    assert crypto_service.encryptedSymmetricyKey.enc == 'A256GCM'
    crypto_service.symmetricKey = json_loads(crypto_service.decrypt(crypto_service.sessionPrivateKey['encodedMuk'], crypto_service.encryptedSymmetricyKey))
    assert crypto_service.symmetricKey['k'] == 'DTCXfnskB1Sm8QiMA_qCWxwZ4GmWOeyAreSN8c0pRcc'
    assert crypto_service.symmetricKey['alg'] == 'A256GCM'
    crypto_service.encryptedAccountKey = Cipher(crypto_service._get_encrypted_account_key())
    assert crypto_service.encryptedAccountKey.enc == 'A256GCM'
    assert crypto_service.encryptedAccountKey.iv == 'Wfvydy9a5MebX9pk'
    assert crypto_service.encryptedAccountKey.data == 'rnU-q-u8Pgb5ZKelOpEZT0HbkjlUCRJ9NqgABLaXhjeXiK1EHOOOYO-l14XC2xr7YBDwQTmG9-kj10KI7yJenOwaR0YIYd-yBkS88_5YilTvQdObGYmnZ6N5DRNZQKQO6ZEY5nJ22QMpMAk5Rt5uwQrB0YXf9MmdJrbMrn0huig-T9GPX6_VwOwYz4WTheq-9QQZJcJRTMszhij0gnPTlKjjagn7ccbdDIJcQsup0AYwhagELZXVhMq6xFh-Sy-6G7dh4JXVWlD5BO2CQv98rD58eDLOjUJlsPsCyMt9X2IwHjaiiQ91kIU9XR5b9MD6v2tPAAOn-5Q7LoyjUXSrCqnT02bvIRGymhO5y4BU-pE7jD76GzPaZ9Lw8MzNOwbXObNR3kadIveaVWxxcf-d2UTmImI3zMoTNvpDlsG17Gl88y2hH1HNHDPZ8K8vPS3GV-N8Y1iv7UYvANigXYYN3iUu2SmtQN0JXfD2lQBlcxhX-4Jzoz1hIlsQ1IpAZ_uMrGYoWOMBJJ69Z3gO6mbLGDk_-fpZwTp82jsvEXverx2Ojan3Q3X80_WX67XSpa5ZyVRl0ipWKkaj7ERjgYo0JEdKni-HCHBdr4M0m1GgtivPGVeRg3ce0Pts7QcWRog'
    crypto_service.accountKey = json_loads(crypto_service.decrypt(crypto_service.symmetricKey['k'], crypto_service.encryptedAccountKey))
    assert crypto_service.accountKey['email'] == 'dev+onepassword-local-search@dkod.fr'
    assert crypto_service.accountKey['masterUnlockKey']['k'] == '2Zqlkn-ppcrz0RaH3wDUiKwu1YUPj1bRM09R9MEmsrE'
    crypto_service.encryptedPrivateKey = Cipher(crypto_service._get_encrypted_private_key())
    assert crypto_service.encryptedPrivateKey.enc == 'A256GCM'
    assert crypto_service.encryptedPrivateKey.iv == 'PKQJjGidIGatDZlC'
    assert crypto_service.encryptedPrivateKey.data == 'Co--8uAZTsy4texj_Tdyli2ENnzJVFT3vplQfQU96-Jn5GpGXiveljeG3ZbZb_snW93EoM8UIFBupMKvs67wIXbL1P8oU6FdK13ypHXkX1DMyIzvg9NXral3ImIS-0MrmzoH_KRLI-CvDXQ3bf2DXYTAA_x2Dh693rnj4Te3h5GZlM0iI-ziFbVnXSuY71QPz8c44OkjcsJx7EPQJyZuryXqwweObnG2sjA449BeUjtDR1tYId8L82oTlWFjvzWsbyOkWoLl-Dsg6Cax7CMoI4QGUVXCCzXRYWia672IhnhD0WziAi_p4abXn8Rb2hd2YwXnY4PFPWA5Esn8nn-046Ga31ibmF7LraFFV5o9kXP6MWcNjHhXjpQHeJ9ptLfl0eLMuJXnMT9Gwxwp71z5_CQaV92AU6Xg686xz5-b1ZUT7ksV6z3UqNK-i8eeOwRJ5ARJwzQRBOyR-A9oYhPwiH_flxEbWzW15hPje5zDMbT5mKQXLRB3hLPltgOcqQsBtY-3KHBWUj2y0ZueBASBYiHnWoAEkIf9u0-gAMVumf4ZHf03EcXm98qV4VCSlDM1uO1W3gNxyg8f3B8F4fmazDxYTVe4kUSVLE5qUqCGxVp-uXS1dgaDwb0T40MDtgnENHMerbSdLg7f7CdRiZtGEJk0ahU6W9LFuAkFxenEALE6-BymFMD_-Z3OYV_ivbQVaC7nwhrDMWfndlcDuZbDpRSwS75iZWs9UiWOrdA3AS6tjWVb9_n_glEYoPG6Nflf_yuAAaZGBI57DsYVSyW_OjNf_5YyYfbAW0Yaptk7r65S5DJB4cKFkbQQtFdHUONe3nQnxexXmirpbIfF97j0nRu8JdqVZsY3X1KcA-abNIq8T0fSi_GrwyMq9qa6JqNRYyrzA5Wau7sT_eSBSu2Ls9UvVRRjISK3oVBbJGhKH5nuHldYsqkGsmyfXKogkjF7PZAS4AgwGUmgLiPtsZvnjsSQdC3K8cyhNYDfmFd5_C3xgxoBEjnpwMTH5_mIo6n_0lwfM7hp-KkQiDnG03bBhauhlw5Af1BfMnFUk6mbEoX3ndSFZ3iF2ckW4QVL-unLKb0eUnjD_9dP6lht7gfgSUSaZoo9QLmLgwD1zQ4iV1bd1R8DRcpUio9lakR5lbjVdRPUcPiwkrmmdtzo1Ve0XM8TLHlo3Bfq8Dh6HYV6ANl4K25GRKCcWjT0n7_-D1qYQSeF2tZDs4IBPD_NbVNmbE_-8jK03n367E87wJ6Vw-wKcUjlB8Q2rMcqGSmp1OQMq1L5EtrXDn_WiOd2JDXaVbo1FyveNC5KVTJGFYDbDzaBkjZFFMzc_ct-uxaiMMrDEiHmwQwvUS1mX5ooi6altsNsxXs855iTAMrDC8wtZYQX8EYkskTLc9sDKYpEUNu8sqxfyd4WtW5F9Pz6KdzDaxmcnma6_SPVTnOnV7pPcyFbG8dF00YwlSPN8RoNgjnqzVoXyNTilj9mRpsnMzonME_sH3_kJ4KdUQfYyEhbtbZLRXnj1VsufB-7p22_B69qy2sKSZFuKK7pbyhHtV2fpSYQtkcgOy1RKt6YbnBc5aV3QZwtAFjVVLAtZPGLt_0J_z3wQJYRGB4q6zBnqWMKipkR-9xEkEKyaG5jy8Y7R87JtPLs_M3vRQ3gvAQmWlYEYmgAJ1wTpc-1cjZ7xgQ8T8ZVGEWYOwH9P9H1ujDsAJkokbedwWjZJbCyZTHmEBs_afaebde0nlrvcNm2CGn7kYJs9VKmME5zoryDaEx934ALKUCe7dSOqvj_498AwYW3Co_H-JPBk19EEKckYmmD86n170UNO8dZMSkqrQufRTT-aRhAVWKXc5yrK68K7MqHVLCqQaxOT78CIRbxKwZbnGLuvdpT6zgp9OJAJPV3nFx_sLWJ1-FhU5sB2Sbiy3d4CdgknXa3HolhpPGnisnqpK1IB5L5VcWHH-uzyN2JY71eiEqD7j4Pvr1flA8QvMPXpmZQl44ri3gkZGi6KpsgIJcatWPi-UfoQuXFWyOa0Kj1SbCMOFURZPZiHZQhaKNpNsQd8m7fxXZGPUAAenfyVp1APqJtk_U8iz92ib_cKqL-soFZSSZBJfV4PKbhHWat5hPwqlH7nrz6iIWc0VGXO-RTeciAWgptU30loMy10DoFj5xxAwCj-_L-gu5oO35p1W3noNJ_sF_2zgxBXIl0TRmwKInQYHdWuTPvdGMsE4p9Bsp_I7UJ7ox_yf0aq7KxMutAg6uaJIwnn5zKnfXlpu9qKUwjm7a8PVMk2MQnH8KRCMwV'
    crypto_service.privateKey = json_loads(crypto_service.decrypt(crypto_service.symmetricKey['k'], crypto_service.encryptedPrivateKey).decode('utf-8'))
    assert crypto_service.privateKey['alg'] == 'RSA-OAEP'
    assert crypto_service.privateKey['d'] == 'AmCxuabUgIyblLBolHSHvZEh_b7PzswAJquPazw9hu6EaN__noNHFIYMYLovfZ99B58XxiuBSo-N7FPMpWGtVTKPBkR1leSp2bxtnz_M25wRquUxJ3BSWy6dNs5pahRu27irQ1b5cTNRrsCm_ew0aviC_7YgbauXgBK3SBmFXRH81Cw_5XCXmO-9Y7TIPwAnd2jThkyyiyZ9KwSyD96h8eqkn6LX_dXYRRnzlCcTzg2TYXWvtynl1S6z4vRmUKmYxmRroFrIukto5Wlq6o0F8nQ_KxvMrud8qMkatuKcaE9LpAzqXHnCsXf7E_n4UKgUCtLkTrQFIqRiJVSs7R_rQQ'


@pytest.mark.usefixtures("op_session")
def test_keyset(crypto_service):
    crypto_service._get_base_keys()
    assert crypto_service.symmetricKey['alg'] == 'A256GCM'
    assert crypto_service.symmetricKey['k'] == 'DTCXfnskB1Sm8QiMA_qCWxwZ4GmWOeyAreSN8c0pRcc'

@pytest.mark.usefixtures("op_session")
def test_private_key(crypto_service):
    crypto_service._get_base_keys()
    assert crypto_service.privateKey['alg'] == 'RSA-OAEP'
    assert crypto_service.privateKey['kid'] == 'vhhhcyj7rc3vocnd5o2iksiflm'
    assert crypto_service.privateKey['n'] == 'vOAZ3IwHI1bkkD5L_5uGvrtMV6KKjzK55ed02Tbqa5Z9k4tKNiwIMykNzmR3XSsNRRthQE6llIh8AJLCbypGnEuCKWZDYmkW_42dF26VjUQ5WqEdWniypsDBcSFqQzBbbb6yv_gs0FNG5QpsEBRuA5DeFSh86CSW-BY35GaARo1G9zDKoqyEI6vZGGX3gv9Nr0docD1Y8ducwkAPtFX4fhFTiBpvJeAGzlKk6imeknZiC4hDFOFt07_vJqORB4Y1crrixPE5E6xkutzgui3WdXBjL14RPIxgfT3_zwc_3Uyy71Y3Tr99MA-i2iWVpRDj9GS3lXh8m9ABp4O4AnU7YQ'

@pytest.mark.usefixtures("op_session")
def test_vault_key(crypto_service):
    vault_key = crypto_service._get_vault_key('6')
    assert vault_key['alg'] == 'A256GCM'
    assert vault_key['k'] == 'CF6mPDqu8Uklk2tfq4Vj5moILWpdxKZycIxQI1VUg1A'

@pytest.mark.usefixtures("op_session")
def test_vault_key(crypto_service):
    item = crypto_service.decrypt_item(Item(crypto_service.storageService.get_item_by_uuid(common_data('item_uuid'))))
    assert item.overview['title'] == 'Software licence'
    assert item.details['sections'][0]['fields'][1]['v'] == 'License number'

@pytest.mark.usefixtures("op_session")
def test_get_encrypted_session_file_path(crypto_service, monkeypatch):
    if os.environ.get('OP_SESSION_PRIVATE_KEY_FILE'):
        monkeypatch.delenv('OP_SESSION_PRIVATE_KEY_FILE')
    crypto_service.sessionKey = crypto_service._get_session_key()
    assert crypto_service._get_encrypted_session_file_path() == os.path.join(os.path.dirname(os.path.dirname(__file__)), 'tests', '.Y_efcm4Gd_W4NnRTMeOuSEHPA5w')